--[[
	FreezeTag
	version 1.0
	by Kiffer-Opa
	
	This script is configurable, scroll down to the word "SETTINGS" to get more details.
]]
if ft==nil then ft={} end

ft.players={}
ft.players.frozen={}
ft.players.melting={}
ft.players.image={}
ft.players.screen={}
ft.players.weapons={}
ft.players.selectedweapon={}

ft.spawnqueue = {}

ft.spawns={}
ft.spawns.t={}
ft.spawns.ct={}
ft.spawns.vip={}

ft.settings={}
ft.settings.melting={}
ft.settings.default={}
ft.settings.gfx={}
ft.settings.hudtxt={}
ft.settings.hudtxt.coords = {}
ft.settings.rewards={}
ft.settings.rewards.money={}
ft.settings.rewards.score={}

--[[ SETTINGS
here you can change some parameters of the game to change the gameplay quickly.
above each parameter there is a small description.
]]

--[[ how fast the melting goes if the team mate stands directly on the frozen player
it should be between 1.00 (immediate melting) and 0.00 (no melting, which does not
make really sense)]]
ft.settings.melting.maxspeed =  0.015

--[[ how close you must be at a frozen player at least to melt him (in pixels)]]
ft.settings.melting.radius   = 42

--[[ the default speedmod of all players (minimum=-100, maximum=100, CS2D default: 0)]]
ft.settings.default.speedmod =  0		

--[[ path to the image for frozen players]]
ft.settings.gfx.ice = "gfx/sprites/flare4.bmp"	

--[[ opaquacity of the ice image (between 1 and 0)]]
ft.settings.gfx.maxalpha = 0.75			

--[[ ice image scale factor
The original size of the image will be multiplied with this factor.]]
ft.settings.gfx.scale = 0.9

--[[ the IDs the hudtxt2 command occupies. This is used for You MUST change this if you use this 
script together with other scripts which use hudtxt with the same
IDs as here. You must use at least one ID but there is no upper limit.]]
ft.settings.hudtxt.ids = { 0,1,2,3,4,5 }	
						
--[[ the screen coordinates where the first "melting ..." message appears (left upper corner)]]
ft.settings.hudtxt.coords.x = 5
ft.settings.hudtxt.coords.y = 100

--[[ how many dollars you get for melting a team mate]]
ft.settings.rewards.money.melt=150

--[[ how many points you get for melting a team mate]]
ft.settings.rewards.score.melt=1

--[[ END OF SETTINGS ]]


function ft.getSpawnpoints()
	for x=0,map("xsize") do
		for y=0,map("ysize") do
			if(entity(x,y,"exists")==true) then
				local typename=entity(x,y,"typename")
				if typename=="Info_T" then
					table.insert(ft.spawns.t,{x=x,y=y})
				elseif typename=="Info_CT" then
					table.insert(ft.spawns.ct,{x=x,y=y})
				elseif typename=="Info_VIP" then
					table.insert(ft.spawns.vip,{x=x,y=y})
				end  
			end
		end
	end
end

function ft.initFreezestates()
	local players=player(0,"table")
	local p
	for i=1,#players do
		p=players[i]
		if(player(p,"team")>0) then	-- if player is not spectating
			if(player(p,"health")>0) then
				ft.makeUnFrozen(p)
			else
				ft.freeze(p)
			end
		end
	end
end

function ft.freeze(pid)
	parse("speedmod "..pid.." -100")
	parse("sethealth "..pid.." "..1)
	local weapons=playerweapons(pid)
	local sv_gamemode=game("sv_gamemode")
	local team=player(pid,"team")
	for i=1,#weapons do
		w=weapons[i]
		-- remove all weapons but knife (human), claw (zombie), bomb, nightvision
		if((sv_gamemode~=4 and w~=50) or (sv_gamemode==4 and (w~=78 and team==1) or (w~=50 and team~=1)) or w~=55 or w~=59) then
			parse("strip "..pid.." "..w)
		end
	end
	parse('effect "flare" '..player(pid,"x")..' '..player(pid,"y")..' 50 16 0 255 255')
	if(ft.players.image[pid]==nil) then
		ft.players.image[pid]=image(ft.settings.gfx.ice,0,0,200+pid)
		imagecolor(ft.players.image[pid], 0,255,255)
		imagealpha(ft.players.image[pid], ft.settings.gfx.maxalpha)
		imageblend(ft.players.image[pid], 1)
		imagescale(ft.players.image[pid], ft.settings.gfx.scale, ft.settings.gfx.scale)
		
		ft.players.screen[pid]=image("gfx/sprites/block.bmp",320,240,2,pid)
		imagecolor(ft.players.screen[pid], 0,255,255)
		imagealpha(ft.players.screen[pid], ft.settings.gfx.maxalpha)
		imageblend(ft.players.screen[pid], 0)
		imagescale(ft.players.screen[pid], 20, 15)
	end
	ft.players.melting[pid]=0.00
	ft.players.frozen[pid]=true
end

function ft.makeUnFrozen(pid)
	parse("speedmod "..pid.." "..ft.settings.default.speedmod)
	parse("sethealth "..pid.." "..player(pid,"maxhealth"))
	if(ft.players.image[pid]~=nil) then
		freeimage(ft.players.image[pid])
		ft.players.image[pid]=nil
	end
	if(ft.players.screen[pid]~=nil) then
		freeimage(ft.players.screen[pid])
		ft.players.screen[pid]=nil
	end
	ft.players.melting[pid]=1.00
	ft.players.frozen[pid]=false
end

function ft.melt(pid)
	ft.makeUnFrozen(pid)
	local weapons=ft.players.weapons[pid]
	if(weapons~=nil) then
		for i=1,#weapons do
			w=weapons[i]
			parse("equip "..pid.." "..w)
			
		end
		parse("setweapon "..pid.." "..ft.players.selectedweapon[pid])
		ft.players.weapons[pid]=nil
	end
end

function ft.isFrozen(pid)
	return ft.players.frozen[pid]
end

function ft.hit(victim,attacker,weapon,hpdmg,apdmg)
	if ft.isFrozen(victim) or ft.isFrozen(attacker) then
		return 1
	else
		return 0
	end
end

function ft.winCondition()
	local players=player(0,"table")
	local frozen_ct, frozen_t, ct, t = 0,0,0,0
	for i=1,#players do
		p=players[i]
		team=player(p,"team")
		if(team==1) then
			t=t+1
			if(ft.isFrozen(p)) then
				frozen_t=frozen_t+1
			end
		elseif(team==2 or team==3) then
			ct=ct+1
			if(ft.isFrozen(p)) then
				frozen_ct=frozen_ct+1
			end
		end
	end
	if(frozen_t == t) then
		ft.killAll(1)
		return 1
	elseif(frozen_ct == ct) then
		ft.killAll(2)
		return 2
	else
		return 0
	end	
end

function ft.killAll(team)
	local players=player(0,"table")
	for i=1,#players do
		local p=players[i]
		pteam=player(p,"team")
		if(pteam==team and player(p,"health")>0) then
			parse("killplayer "..p)
			parse("setdeaths "..p.." "..(player(p,"deaths")-1))
		end
	end
end

function ft.melting()
	local players=player(0,"table")
	local hudtxt_p2={}
	for i=1,32 do
		hudtxt_p2[i]={}
	end
	for i=1,#players do
		local p1=players[i]
		local melters = 0
		for j=1,#players do
			local p2=players[j]
			
			if(ft.players.frozen[p1]==true and ft.players.frozen[p2]==false and player(p2,"health")>0) then
				-- FIXME: fails on CT compared with VIP
				if(player(p1,"team")==player(p2,"team")) then
					if(ft.distance(p1,p2) <= ft.settings.melting.radius) then
						local dist = (ft.settings.melting.radius-ft.distance(p1,p2))/ft.settings.melting.radius
						ft.players.melting[p1] = ft.players.melting[p1] + ft.settings.melting.maxspeed * dist
						if(ft.players.melting[p1]>=1.0) then
							ft.melt(p1)
							parse('setscore '..p2..' '..(player(p2,"score")+1))
							parse('setmoney '..p2..' '..(player(p2,"money")+150))
							parse('hudtxt2 '..p1..' '..ft.settings.hudtxt.ids[1])
						else
							melters = melters + 1
							percentage = string.format("%.1f",ft.players.melting[p1] * 100)
							text="melting "..player(p1,"name").." ("..percentage.."%)"
							imagealpha(ft.players.image[p1], (1-ft.players.melting[p1])*ft.settings.gfx.maxalpha)
							imagealpha(ft.players.screen[p1], (1-ft.players.melting[p1])*ft.settings.gfx.maxalpha)
							table.insert(hudtxt_p2[p2],text)
						end
					end
				end
			end
		end
		
		if(ft.players.frozen[p1]==true and player(p1,"health")>0) then
			if(melters==0) then
				ft.players.melting[p1] = ft.players.melting[p1] - ft.settings.melting.maxspeed
				if(ft.players.melting[p1]<0) then
					ft.players.melting[p1]=0
				end
			end		
			if(ft.players.melting[p1]>=0.001) then
			percentage = string.format("%.1f",ft.players.melting[p1] * 100)
				text="melting ... ("..percentage.."%)"
				imagealpha(ft.players.image[p1], (1-ft.players.melting[p1])*ft.settings.gfx.maxalpha)
				imagealpha(ft.players.screen[p1], (1-ft.players.melting[p1])*ft.settings.gfx.maxalpha)
				parse('hudtxt2 '..p1..' '..ft.settings.hudtxt.ids[1]..' "'..text..'" '..ft.settings.hudtxt.coords.x..' '..ft.settings.hudtxt.coords.y..' 0')
			else
				parse('hudtxt2 '..p1..' '..ft.settings.hudtxt.ids[1])
			end
		else
			parse('hudtxt2 '..p1..' '..ft.settings.hudtxt.ids[1])
		end

	end
	
	for i=1,32 do
		if(hudtxt_p2[i]~=nil) then
			if(#ft.settings.hudtxt.ids < #hudtxt_p2[i]) then
				max=1
				parse('hudtxt2 '..i..' '..ft.settings.hudtxt.ids[1]..' "melting '..#hudtxt_p2[i]..' players" '..ft.settings.hudtxt.coords.x..' '..ft.settings.hudtxt.coords.y..' 0')
			else
				max=#ft.settings.hudtxt.ids
				for j=1,#hudtxt_p2[i] do
					parse('hudtxt2 '..i..' '..ft.settings.hudtxt.ids[j]..' "'..hudtxt_p2[i][j]..'" '..ft.settings.hudtxt.coords.x..' '..ft.settings.hudtxt.coords.y+((j-1)*12)..' 0')
					max=j
				end
			end
			for j=max+1,#ft.settings.hudtxt.ids do
				parse('hudtxt2 '..i..' '..ft.settings.hudtxt.ids[j])
			end
		end
	end
end

function ft.distance(p1,p2)
	local x1,y1,x2,y2
	x1=player(p1,"x")
	y1=player(p1,"y")
	x2=player(p2,"x")
	y2=player(p2,"y")
	return math.sqrt(math.pow(math.abs(x2-x1),2)+math.pow(math.abs(y2-y1),2))
end

function ft.buildattempt(pid,buildingtype,x,y)
	if(ft.isFrozen(pid)) then
		parse('sv_msg2 '..pid..[[ "255000000You can't build while you're frozen.@C"]])
		return 1
	else
		return 0
	end
end

function ft.drop(pid,iid,itemtype,ain,a,mode,x,y)
	if(ft.isFrozen(pid)) then
		parse('sv_msg2 '..pid..[[ "255000000You can't drop anything while frozen.@C"]])
		return 1
	else
		return 0
	end
end

function ft.buy(pid,weapon)
	if(ft.isFrozen(pid)) then
		parse('sv_msg2 '..pid..[[ "255000000You can't buy while you're frozen.@C"]])
		return 1
	else
		return 0
	end
end

function ft.die(victim,killer,weapon,x,y)
	tilex=math.floor(x/32)
	tiley=math.floor(y/32)
	if(ft.winCondition()==0) then
		table.insert(ft.spawnqueue, { pid=victim, x=x, y=y })
		weapons=playerweapons(victim)
		for i=#weapons,1,-1 do
			w=weapons[i]
			-- remove flag and bomb
			if(w==70 or w==71 or w==55) then
				table.remove(weapons,i)
			end
		end
		ft.players.weapons[victim]=playerweapons(victim)
		
		ft.players.selectedweapon[victim]=player(victim,"weapontype")
		return 1
	else
		if(ft.players.screen[victim]~=nil) then
			freeimage(ft.players.screen[victim])
			ft.players.screen[victim]=nil
		end
		return 0
	end
end

function ft.respawnAfterFreeze()
	for i=#ft.spawnqueue,1,-1 do
		local pl=ft.spawnqueue[i]
		local tilex=math.floor(pl.x/32)
		local tiley=math.floor(pl.y/32)
		if(tile(tilex,tiley,"walkable")==true and tile(tilex,tiley,"deadly")==false) then
			if(player(pl.pid,"team")>0) then
				parse("spawnplayer "..pl.pid.." "..pl.x.." "..pl.y)
			end
		else
			if(player(pl.pid,"team")>0) then
				ft.normalSpawn(pl.pid,player(pl.pid,"team"))
			end
		end
		ft.freeze(pl.pid)
		table.remove(ft.spawnqueue,i)
	end
	
end

function ft.normalSpawn(pid,team)
	if(team==nil) then
		team=player(pid,"team")
	end
	local spawnpoint,x,y
	
	if(team==1) then
		spawnpoint=math.random(1,#ft.spawns.t)
		x=ft.spawns.t[spawnpoint].x
		y=ft.spawns.t[spawnpoint].y
	elseif(team==2) then
		spawnpoint=math.random(1,#ft.spawns.ct)
		x=ft.spawns.ct[spawnpoint].x
		y=ft.spawns.ct[spawnpoint].y
	elseif(team==3) then
		spawnpoint=math.random(1,#ft.spawns.vip)
		x=ft.spawns.vip[spawnpoint].x
		y=ft.spawns.vip[spawnpoint].y
	else
		return
	end
	parse("spawnplayer "..pid.." "..(x*32+16).." "..(y*32+16))
end


function ft.spawn(pid)
	if(ft.isFrozen(pid)) then
		return "x"
	else
		return ""
	end
end

function ft.initHudtxt()
	for p=1,32 do
		for i=1,#ft.settings.hudtxt.ids do
			parse("hudtxt2 "..p.." "..ft.settings.hudtxt.ids[i])
		end
	end
end

function ft.team(pid,team,look)
	if(team==0) then
		if(ft.players.screen[pid]~=nil) then
			freeimage(ft.players.screen[pid])
			ft.players.screen[pid]=nil
		end
	else
		table.insert(ft.spawnqueue,{pid=pid,x=-32,y=-32})
	end
	return 0
end

addhook("hit","ft.hit")
addhook("startround","ft.initFreezestates")
addhook("startround","ft.initHudtxt")
addhook("second","ft.winCondition")
addhook("always","ft.melting")
addhook("always","ft.respawnAfterFreeze")
addhook("buildattempt","ft.buildattempt")
addhook("buy","ft.buy")
addhook("die","ft.die")
addhook("spawn","ft.spawn")
addhook("team","ft.team")
ft.initFreezestates()
ft.getSpawnpoints()